using System;
using System.Collections.Generic;
using System.Timers;

namespace events
{
  /// <summary>
  /// method definition
  /// </summary>
  internal delegate void LightHandler(Light sender);

  /// <summary>
  /// Pouziti eventu. K definovani eventu je treba:
  /// A) definovat delegate
  /// B) definovat event
  /// C) vyvolat event uvnitr tridy
  /// </summary>
  internal class TraficLight
  {
    private Timer timer;
    int lastIndex = 0;
    List<Light> lights;

    /// <summary>
    /// Vyvolan pokud se zmeni svetlo.
    /// Pouzij k registrovani vlastniho posluchace
    /// </summary>
    public event LightHandler StatusChange;

    /// <summary>
    /// Default constructor implementation to start timer.
    /// </summary>
    public TraficLight()
    {
      lights = new List<Light>(); //initialize lights
      Light lightA = new Light();
      lightA.Color = ConsoleColor.Red;
      lights.Add(lightA);

      Light lightB = new Light();
      lights.Add(lightB);

      Light lightC = new Light();
      lightC.Color = ConsoleColor.Green;
      lights.Add(lightC);


      timer = new Timer(); //initializace timeru
      timer.Interval = 2000;
      timer.Elapsed += new ElapsedEventHandler(timer_Elapsed); //take pouziti event handleru
      timer.Start();
    }

    private void timer_Elapsed(object sender, ElapsedEventArgs e)
    {
      lights[lastIndex].SwitchOff();
      lastIndex++;
      if (lastIndex > 2) lastIndex = 0;
      Light newLight = lights[lastIndex];
      newLight.SwitchOn();

      if (StatusChange != null) 
      {
        StatusChange(newLight); //vyvolej event pouze pokud je registrovan alespon jeden posluchac
      }
    }
  }
}
